require 'aws-sdk'
require 'logger'

module MistAws
  class Ec2

    # These are read-only accessor and are initializeds by initialize method
    attr_reader   :profile_name
    attr_reader   :credentials
    attr_reader   :region
    attr_reader   :logger
    attr_reader   :iam
    attr_reader   :ec2_client
    attr_reader   :ec2

    def initialize(opts={})
      # Ruby 1.9 backwards compatability
      opts = {profile_name: nil, region: nil, logger: ::Logger.new(STDERR)}.merge(opts)
      opts.each do |key, value|
        instance_variable_set "@#{key}", value
      end

      @iam = Iam.new(opts)
      @ec2_client = Aws::EC2::Client.new(credentials: @iam.credentials, region: @iam.region)
      @ec2 = Aws::EC2::Resource.new(client: @ec2_client)

    end

    def get_security_group(group_name, vpc_id)
      ec2_client.describe_security_groups(filters: [{ name: "vpc-id", values: [vpc_id]}]).security_groups.detect { |g| g.group_name == group_name }
    end

    def get_security_group_id(group_name, vpc_id)
      if (group = get_security_group(group_name, vpc_id))
        group.group_id
      else
        nil
      end
    end

    def security_group_exists?(group_name, vpc_id)
      get_security_group(group_name, vpc_id)
    end

    def delete_security_group(group_name, vpc_id)
      group_id = get_security_group_id(group_name, vpc_id)
      ec2_client.delete_security_group(group_id: group_id) if group_id
    end

    def create_security_group(group_name, vpc_id, description=group_name)
      begin
        ec2_client.create_security_group(group_name: group_name, vpc_id: vpc_id, description: description)
      rescue Aws::EC2::Errors::InvalidGroupDuplicate
      end
      # Seem to need to fetch it to get a valid security group struct
      get_security_group(group_name, vpc_id)
    end

    # Note the keys for opts must be strings not symbols 
    def authorize_security_group_ingress(opts={})
      begin
        ec2_client.authorize_security_group_ingress(opts)
      rescue Aws::EC2::Errors::InvalidPermissionDuplicate
      end
    end

    def create_vpc(cdr_block)
      result = ec2_client.describe_vpcs(filters: [{ name: "cidr", values: [cdr_block]}])
      if result.vpcs == []
        ec2_client.create_vpc(cidr_block: cdr_block).vpc
      else
        result.vpcs.first
      end
    end

    def delete_vpc(vpc_id)
      begin
        ec2_client.delete_vpc(vpc_id: vpc_id)
      rescue Aws::EC2::Errors::InvalidVpcIDNotFound
        nil
      end
    end

  end
end

